searchNodes=[{"doc":"Gradualizer is a static type checker for Erlang with support for gradual typing . A static type checker detects errors thanks to type analysis of a given program. Gradualizer can infer some types, but mostly relies on function specs. In a nutshell, we could say that it checks for consistency of a function body with its declared spec and for consistency of a callee's spec with passed in arguments. Specifically, Gradualizer does not perform entire program flow analysis. Gradual typing means that full type information is not required to raise warnings - any type information is better than no type information. It allows for a _gradual_ transition between fully static typing, which offers the best correctness guarantee, and fully dynamic typing, which means the least overhead of writing specs, the full Erlang expressiveness, and _letting it crash_ at runtime. We can choose the right balance between the static/dynamic extremes depending on the application, project maturity, team size, the need for reusability or for documentation consistency. If you are interested in using Gradualizer to type check your code, then you might be looking for the Rebar3 plugin rebar_prv_gradualizer or the command-line interface gradualizer_cli . This module contains entry points for calling into Gradualizer as a library, as well as some Erlang shell utilities for working directly with the type checker internals. Let's try out the shell utilities by running: $ rebar3 shell Now, we can play with some examples: &gt; gradualizer : type_of ( &quot;[ a || _ &lt;- lists:seq(1, 5) ]&quot; ) . { type , 0 , list , [ { atom , 0 , a } ] } &gt; typelib : pp_type ( v ( - 1 ) ) . &quot;[a]&quot; &gt; typechecker : normalize ( gradualizer : type ( &quot;a()&quot; ) , gradualizer : env ( &quot;-type a() :: integer().&quot; , [ ] ) ) . { type , 0 , integer , [ ] } &gt; typelib : pp_type ( v ( - 1 ) ) . &quot;integer()&quot; &gt; gradualizer : type_of ( &quot;fun (A) -&gt; \#{tag =&gt; my_map, list_of_as =&gt; [ A || _ &lt;- lists:seq(1, 5) ]} end&quot; ) . { type , 0 , &#39;fun&#39; , [ { type , 0 , product , [ { type , 0 , any , [ ] } ] } , { type , 0 , map , [ { type , 0 , map_field_assoc , [ { atom , 0 , tag } , { atom , 0 , my_map } ] } , { type , 0 , map_field_assoc , [ { atom , 0 , list_of_as } , { type , 0 , list , [ { type , 0 , any , [ ] } ] } ] } ] } ] } &gt; typelib : pp_type ( v ( - 1 ) ) . &quot;fun((any()) -&gt; \#{tag =&gt; my_map, list_of_as =&gt; [any()]})&quot; The main library API of Gradualizer are type_check_(file|module|dir) functions, which accept options described by the options() type.","ref":"gradualizer.html","title":"gradualizer","type":"module"},{"doc":"See also: env/2 .","ref":"gradualizer.html#env/0","title":"gradualizer.env/0","type":"function"},{"doc":"See also: env/2 .","ref":"gradualizer.html#env/1","title":"gradualizer.env/1","type":"function"},{"doc":"Create a type checker environment populated by types defined in a source code snippet via -type ... and -record(...) attributes. Currently, it's not possible to define variable bindings in the environment. &gt; rr ( typechecker ) . &gt; gradualizer : env ( &quot;-type a() :: integer().&quot; , [ ] ) . # env { fenv = \#{ } , imported = \#{ } , venv = \#{ } , tenv = \#{ module =&gt; undefined , records =&gt; \#{ } , types =&gt; \#{ { a , 0 } =&gt; { [ ] , { type , 0 , integer , [ ] } } } } , infer = false , verbose = false , exhaust = true , clauses_stack = [ ] , union_size_limit = 30 , current_spec = none } &gt; gradualizer : env ( &quot;-record(r, {f}).&quot; , [ ] ) . # env { fenv = \#{ } , imported = \#{ } , venv = \#{ } , tenv = \#{ module =&gt; undefined , records =&gt; \#{ r =&gt; [ { typed_record_field , { record_field , 0 , { atom , 0 , f } , { atom , 1 , undefined } } , { type , 0 , any , [ ] } } ] } , types =&gt; \#{ } } , infer = false , verbose = false , exhaust = true , clauses_stack = [ ] , union_size_limit = 30 , current_spec = none }","ref":"gradualizer.html#env/2","title":"gradualizer.env/2","type":"function"},{"doc":"Return a Gradualizer type for the passed in Erlang type definition. &gt; gradualizer : type ( &quot;[integer()]&quot; ) . { type , 0 , list , [ { type , 0 , integer , [ ] } ] }","ref":"gradualizer.html#type/1","title":"gradualizer.type/1","type":"function"},{"doc":"Type check all source or beam files in a directory.","ref":"gradualizer.html#type_check_dir/1","title":"gradualizer.type_check_dir/1","type":"function"},{"doc":"Type check all source or beam files in a directory.","ref":"gradualizer.html#type_check_dir/2","title":"gradualizer.type_check_dir/2","type":"function"},{"doc":"Type check a source or beam file","ref":"gradualizer.html#type_check_file/1","title":"gradualizer.type_check_file/1","type":"function"},{"doc":"Type check a source or beam file","ref":"gradualizer.html#type_check_file/2","title":"gradualizer.type_check_file/2","type":"function"},{"doc":"Type check a source or beam file","ref":"gradualizer.html#type_check_files/1","title":"gradualizer.type_check_files/1","type":"function"},{"doc":"Type check a source or beam","ref":"gradualizer.html#type_check_files/2","title":"gradualizer.type_check_files/2","type":"function"},{"doc":"Type check an abstract syntax tree of a module. This can be useful for tools where the abstract forms are generated in memory. If the first form is a file attribute (as in forms returned by e.g. epp:parse_file/1,2), that filename will be used in error messages. The second form is typically the module attribute.","ref":"gradualizer.html#type_check_forms/2","title":"gradualizer.type_check_forms/2","type":"function"},{"doc":"Type check a module","ref":"gradualizer.html#type_check_module/1","title":"gradualizer.type_check_module/1","type":"function"},{"doc":"Type check a module","ref":"gradualizer.html#type_check_module/2","title":"gradualizer.type_check_module/2","type":"function"},{"doc":"See also: type_of/2 .","ref":"gradualizer.html#type_of/1","title":"gradualizer.type_of/1","type":"function"},{"doc":"Infer type of an Erlang expression. &gt; gradualizer : type_of ( &quot;[ a || _ &lt;- lists:seq(1, 5) ]&quot; ) . { type , 0 , list , [ { atom , 0 , a } ] } &gt; gradualizer : type_of ( &quot;case 5 of 1 -&gt; one; _ -&gt; more end&quot; ) . { type , 0 , union , [ { atom , 0 , more } , { atom , 0 , one } ] } &gt; Env = gradualizer : env ( [ { union_size_limit , 1 } ] ) . &gt; gradualizer : type_of ( &quot;case 5 of 1 -&gt; one; _ -&gt; more end&quot; , Env ) . { type , 0 , any , [ ] }","ref":"gradualizer.html#type_of/2","title":"gradualizer.type_of/2","type":"function"},{"doc":"The options accepted by the Gradualizer API. {i, Dir} : include path for -include and -include_lib when checking Erlang source files. Specify multiple times for multiple include paths. stop_on_first_error : if true stop type checking on the first error, if false continue checking all functions in the given file and all files in the given directory. crash_on_error : if true crash on the first produced error. return_errors : return a list of errors instead of printing them and returning a single condensed ok | nok . fmt_location : how to format location when pretty printing errors: none : no location for easier comparison brief : for machine processing ( &lt;line&gt;:&lt;column&gt;: before message text) verbose (default): for human readers ( on line &lt;line&gt; at column &lt;column&gt; within the message text) fmt_expr_fun : function to pretty print an expression AST (useful to support other languages) fmt_type_fun : function to pretty print a type AST (useful to support other languages) {color, always | never | auto} : Use colors when printing fancy messages. Auto is the default but auto-detection of tty doesn't work when running as an escript. It works when running from the Erlang shell though. {fancy, boolean()} : Use fancy error messages when possible. True by default. Doesn't work when a custom fmt_expr_fun is used.","ref":"gradualizer.html#t:options/0","title":"gradualizer.options/0","type":"type"},{"doc":"","ref":"gradualizer.html#t:top/0","title":"gradualizer.top/0","type":"opaque"},{"doc":"Gradualizer command line interface.","ref":"gradualizer_cli.html","title":"gradualizer_cli","type":"module"},{"doc":"","ref":"gradualizer_cli.html#handle_args/1","title":"gradualizer_cli.handle_args/1","type":"function"},{"doc":"","ref":"gradualizer_cli.html#main/1","title":"gradualizer_cli.main/1","type":"function"},{"doc":"A utility module to make troubleshooting problems by tracing Gradualizer internals more efficient. Configuration of this module is compile time, but using the tracing facilities is more efficient than traditional printf-debugging anyway. Together with the shell interface of gradualizer , we can use this module to inspect or debug the inner workings of the type checker live. Watch out for the trace sizes - they can grow huge! &gt; gradualizer_tracer : start ( ) . ok &gt; gradualizer : type_of ( &quot;[a]&quot; ) . { trace , &lt; 0.1296 . 0 &gt; , call , { typechecker , type_check_expr , [ { venv , \#{ } } , { cons , 1 , { atom , 1 , a } , { nil , 1 } } ] } } { trace , &lt; 0.1296 . 0 &gt; , call , { typechecker , type_check_expr , [ { venv , \#{ } } , { atom , 1 , a } ] } } { trace , &lt; 0.1296 . 0 &gt; , return_from , { typechecker , type_check_expr , 2 } , { { atom , 0 , a } , { venv , \#{ } } , { constraints , \#{ } , \#{ } , \#{ } } } } { trace , &lt; 0.1296 . 0 &gt; , return_to , { typechecker , do_type_check_expr , 2 } } { trace , &lt; 0.1296 . 0 &gt; , call , { typechecker , type_check_expr , [ { venv , \#{ } } , { nil , 1 } ] } } { trace , &lt; 0.1296 . 0 &gt; , return_from , { typechecker , type_check_expr , 2 } , { { type , 0 , nil , [ ] } , { venv , \#{ } } , { constraints , \#{ } , \#{ } , \#{ } } } } { trace , &lt; 0.1296 . 0 &gt; , return_to , { typechecker , do_type_check_expr , 2 } } { trace , &lt; 0.1296 . 0 &gt; , return_from , { typechecker , type_check_expr , 2 } , { { type , 0 , nonempty_list , [ { atom , 0 , a } ] } , { venv , \#{ } } , { constraints , \#{ } , \#{ } , \#{ } } } } { trace , &lt; 0.1296 . 0 &gt; , return_to , { g , type_of , 2 } } { type , 0 , nonempty_list , [ { atom , 0 , a } ] }","ref":"gradualizer_tracer.html","title":"gradualizer_tracer","type":"module"},{"doc":"debug/1 is a trace point to trace when pinpointing issues across several candidate locations. Uncomment the below in start/0 : dbg : tpl ( ? MODULE , debug , x ) Then insert the following somewhere in code, recompile and check the trace for the line from which debug/1 was called: gradualizer_tracer : debug ( ? LINE )","ref":"gradualizer_tracer.html#debug/1","title":"gradualizer_tracer.debug/1","type":"function"},{"doc":"Wait for flushing all the trace messages to stdout.","ref":"gradualizer_tracer.html#flush/0","title":"gradualizer_tracer.flush/0","type":"function"},{"doc":"Start tracing. Check out the source code of this function to see trace pattern examples matching specific parts of the type checker. Uncomment the relevant ones or add your own, recompile the project, and start tracing.","ref":"gradualizer_tracer.html#start/0","title":"gradualizer_tracer.start/0","type":"function"},{"doc":"Stop tracing.","ref":"gradualizer_tracer.html#stop/0","title":"gradualizer_tracer.stop/0","type":"function"},{"doc":"The Rebar3 plugin for Gradualizer. To run Gradualizer from Rebar3, add it as a plugin in your rebar.config : { plugins , [ { gradualizer , { git , &quot;git@github.com:josefs/Gradualizer.git&quot; , { branch , &quot;master&quot; } } } ] } .","ref":"rebar_prv_gradualizer.html","title":"rebar_prv_gradualizer","type":"module"},{"doc":"","ref":"rebar_prv_gradualizer.html#do/1","title":"rebar_prv_gradualizer.do/1","type":"function"},{"doc":"","ref":"rebar_prv_gradualizer.html#format_error/1","title":"rebar_prv_gradualizer.format_error/1","type":"function"},{"doc":"","ref":"rebar_prv_gradualizer.html#init/1","title":"rebar_prv_gradualizer.init/1","type":"function"},{"doc":"Example user_default file to extend your Erlang shell with Gradualizer features.","ref":"user_default.html","title":"user_default","type":"module"},{"doc":"Type check and compile file. See also: //stdlib/shell_default:c/1.","ref":"user_default.html#c/1","title":"user_default.c/1","type":"function"},{"doc":"","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Gradualizer is a static type checker for Erlang with some support for gradual typing . A static type checker catches bugs before we run a program thanks to type analysis. Gradualizer aims to integrate well into existing Erlang code bases in a non intrusive way. It does so by having a type system that is based on the principles of Gradual Typing using the existing Erlang type spec syntax allowing for granular opting into type checking. Without any type specs, no static typing happens. When type specs are added the program is checked against these specs statically. The more type specs, the more static typing. A picture is worth more than a thousand words:","ref":"readme.html#a-type-checker-for-erlang","title":"Overview - A type checker for Erlang","type":"extras"},{"doc":"Command line Compile the project as an escript. Then use it to check .beam files or .erl files. Use the -h / --help option for help. $ make escript $ bin/gradualizer --help Usage: gradualizer [options] [PATH...] A type checker for Erlang/Elixir PATH Files or directories to type check -- Signals that no more options will follow. The following arguments are treated as filenames, even if they start with hyphens. -h, --help display this help and exit --infer Infer type information from literals and other language constructs --no_infer Only use type information from function specs - the default behaviour --verbose Show what Gradualizer is doing -pa, --path_add Add the specified directory to the beginning of the code path; see erl -pa [string] -I Include path for Erlang source files; see -I in the manual page erlc(1) --stop_on_first_error stop type checking at the first error --no_stop_on_first_error inverse of --stop-on-first-error - the default behaviour --no_prelude Do not override OTP specs. --specs_override_dir Add specs overrides from the *.specs.erl files in this directory. --fmt_location How to format location when pretty printing errors (Column is only available if analyzing from source) - &#39;none&#39;: no location for easier comparison - &#39;brief&#39;: for machine processing (&quot;LINE:COLUMN:&quot; before message text) - &#39;verbose&#39; (default): for human readers (&quot;on line LINE at column COLUMN&quot; within the message text) --color [ COLOR ] - Use colors when printing fancy messages. An optional argument is `always | never | auto&#39;. However, auto- detection of a TTY doesn&#39;t work when running as an escript. --no_color - Alias for `--color never&#39; --fancy - Use fancy error messages when possible (on by default) --no_fancy - Don&#39;t use fancy error messages. --union_size_limit - Performance hack: Unions larger than this value are replaced by any() in normalization (default: 30) Rebar3 To run Gradualizer from rebar3, add it as a plugin in your rebar.config : { plugins , [ { gradualizer , { git , &quot;git@github.com:josefs/Gradualizer.git&quot; , { branch , &quot;master&quot; } } } ] } . See examples/rebar3/README.md . Elixir / Mix Check out Gradient , the Elixir frontend to Gradualizer. It provides a Mix task to use in your project: def deps do [ { :gradient , github : &quot;esl/gradient&quot; , only : [ :dev ] , runtime : false } ] end Erlang shell Launch the interactive prompt with all the relevant modules in the path. Then, use the functions in the gradualizer module: $ make shell 1&gt; gradualizer:type_check_file(&quot;path/to/some_file.erl&quot;). You can also use the Rebar3 shell. Prerequisites Gradualizer requires at least OTP 21 and is built using plain OTP functionality and a self-contained Makefile. Alternatively, it can be built using rebar3 , as well as Mix if used as a dependency.","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"Gradualizer is close to a beta release. Most of the Erlang language constructs and data types are handled, although there are things that don't work yet. That being said, pull requests are most welcome! As of writing this, the single biggest missing feature is support for subtype polymorphism. A work-in-progress Gradualizer manual is located on the wiki. For a non-exhaustive list of known problems, see test/known_problems/ .","ref":"readme.html#status","title":"Overview - Status","type":"extras"},{"doc":"MIT License Copyright (c) 2021-2022 Radek Szymczyszyn Copyright (c) 2017-2019 Josef Svenningsson Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"}]